/*
 * write_nav.c - functions to write NAV (FPLAN) format output
 *
 * Copyright (C) 1998, John C. Peterson <mailto:jaypee@netcom.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2. A copy is included in this distribution in the file
 * named "LICENSE".
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License, version 2, for more details.
 *
 * $Id: write_nav.c,v 1.1 1998/05/05 06:44:07 jcp Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "faaconv.h"

static char outbuf[BUF_LEN];

static bool_t write_nav_apt(char **tok_str);
static bool_t write_nav_fix(char **tok_str);
static bool_t write_nav_ils(char **tok_str);
static bool_t write_nav_nav(char **tok_str);
static bool_t write_nav_rwy(char **tok_str);

/* -------------------------------------------------------------------------- */

static bool_t write_nav_apt(char **tok_str)
{

  apt_t *rec = (void *) tok_str;

  char *p = outbuf;

  int nscan;
  int degrees;
  int lat_deg;
  int lon_deg;
  int minutes;
  double lat_min;
  double lon_min;
  double var_min;
  double seconds;
  char lat_ns[2];
  char lon_ew[2];
  char var_ew[2];

  /* airport id */

  if (option_k) {
    rec->airport_id = strkid(rec->airport_id);
  }

  if (rec->airport_id[0] != '\0') {
    if (strlen(rec->airport_id) > 5)
      rec->airport_id[5] = '\0';
    SPRINTF(p,"%s:", rec->airport_id);
  } else {
    return FALSE;
  }

  /* city, state code */

  rec->city = strcap(rec->city);
  if (strlen(rec->city) > 28)
    rec->city[28] = '\0';
  SPRINTF(p,"%s", rec->city);
  if (strlen(rec->state_post_office_code) > 2)
    rec->state_post_office_code[2] = '\0';
  SPRINTF(p,", %s:", rec->state_post_office_code);

  /* airport name */

  if (strlen(rec->airport_name) > 32)
    rec->airport_name[32] = '\0';
  SPRINTF(p,"%s:", rec->airport_name);

  /* airport elevation (feet) */

  SPRINTF(p,"%s:", rec->airport_elevation);

  /* scan latitude */

  nscan = sscanf(rec->ref_point_latitude, "%d-%d-%lf%1[NS]",
                 &lat_deg, &minutes, &seconds, lat_ns);

  if (nscan == 4) {
    lat_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* scan longitude */

  nscan = sscanf(rec->ref_point_longitude, "%d-%d-%lf%1[EW]",
                 &lon_deg, &minutes, &seconds, lon_ew);

  if (nscan == 4) {
    lon_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* signed magnetic variation */

  if (model_priority) {

    /* use the geomagnetic field model to compute the magnetic variation */

    model.latitude = ((*lat_ns == 'N')? +1.0: -1.0)*(lat_deg + lat_min/60.0);
    model.longitude = ((*lon_ew == 'E')? +1.0: -1.0)*(lon_deg + lon_min/60.0);
    if (sscanf(rec->airport_elevation, "%lf", &model.altitude) == 1) {
      model.altitude = FEET2KM(model.altitude);
    } else {
      model.altitude = 1.0;
    }

    if (field_model(&model)) {
      degrees = fabs(model.dec);
      var_min = (fabs(model.dec) - degrees)*60.0;
      if (model.dec > 0.0) SPRINTF(p,"%s", "-");
      SPRINTF(p,"%d:", degrees);
      SPRINTF(p,"%g:", var_min);
    } else {
      return FALSE;
    }

  } else {

    /* get the magnetic variation from the database */

    nscan = sscanf(rec->mag_variation_and_direction_deg,
                   "%d%1[EW]", &degrees, var_ew);

    if (nscan == 2) {
      if (*var_ew == 'E') SPRINTF(p,"%s", "-");
      SPRINTF(p,"%d:0:", degrees);
    } else {
      return FALSE;
    }

  }

  /* signed latitude */

  if (*lat_ns == 'S') SPRINTF(p,"%s", "-");
  SPRINTF(p,"%d:", lat_deg);
  SPRINTF(p,"%g:", lat_min);

  /* signed longitude */

  if (*lon_ew == 'E') SPRINTF(p,"%s", "-");	/* bass ackwards convention! */
  SPRINTF(p,"%d:", lon_deg);
  SPRINTF(p,"%g:", lon_min);

  /* common traffic advisory frequency */

  SPRINTF(p,"%s:", rec->common_traffic_advisory_frequency);

  /* comment string */

  if (rec->airport_type[0] != '\0') {
    if (strcmp(rec->airport_ownership_type,"PR") == 0) {
      SPRINTF(p,"%s ", "Pvt.");
    }
    SPRINTF(p,"%s", strcap(rec->airport_type));
  }

  if (rec->aeronautical_sectional_chart_for_airport[0] != '\0') {
    SPRINTF(p,", %s Sect.",
              strcap(rec->aeronautical_sectional_chart_for_airport));
  }

  if (rec->airport_attendance_schedule[0] != '\0') {
    SPRINTF(p,", %s", rec->airport_attendance_schedule);
  }

  if (rec->fuel_types_available_for_public_use[0] != '\0') {
    SPRINTF(p,", %s", rec->fuel_types_available_for_public_use);
  }

  if (rec->type_of_bottled_oxygen_available[0] != '\0') {
    if (strcmp(rec->type_of_bottled_oxygen_available,"NONE") != 0) {
      SPRINTF(p,", %s", rec->type_of_bottled_oxygen_available);
    }
  }

  if (rec->type_of_bulk_oxygen_available[0] != '\0') {
    if (strcmp(rec->type_of_bulk_oxygen_available,"NONE") != 0) {
      SPRINTF(p,", %s", rec->type_of_bulk_oxygen_available);
    }
  }

  /* normal exit */

  *p = '\0';

  fprintf(fpoutput, "%s\n", outbuf);

  return TRUE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_nav_fix(char **tok_str)
{

  fix_t *rec = (void *) tok_str;

  char *p = outbuf;

  int nscan;
  int degrees;
  int lat_deg;
  int lon_deg;
  int minutes;
  double lat_min;
  double lon_min;
  double var_min;
  double seconds;
  char lat_ns[2];
  char lon_ew[2];
  char *navaid_id;
  char *radial_dist;

  /* reject everything but reporting points */

  if (strcmp(rec->fix_use,"REP-PT") != 0) {
    return FALSE;
  }

  /* fix name */

  if (rec->fix_name[0] != '\0') {
    if (strlen(rec->fix_name) > 5)
      rec->fix_name[5] = '\0';
    SPRINTF(p,"%s:", rec->fix_name);
  } else {
    return FALSE;
  }

  /* facility name */

  SPRINTF(p,"%s", ":");

  /* frequency */

  SPRINTF(p,"%s", ":");

  /* elevation (feet) */

  SPRINTF(p,"%s", ":");

  /* scan latitude */

  nscan = sscanf(rec->fix_latitude, "%d-%d-%lf%1[NS]",
                 &lat_deg, &minutes, &seconds, lat_ns);

  if (nscan == 4) {
    lat_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* scan longitude */

  nscan = sscanf(rec->fix_longitude, "%d-%d-%lf%1[EW]",
                 &lon_deg, &minutes, &seconds, lon_ew);

  if (nscan == 4) {
    lon_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* signed magnetic variation */

  model.latitude = ((*lat_ns == 'N')? +1.0: -1.0)*(lat_deg + lat_min/60.0);
  model.longitude = ((*lon_ew == 'E')? +1.0: -1.0)*(lon_deg + lon_min/60.0);
  model.altitude = 1.0;

  if (field_model(&model)) {
    degrees = fabs(model.dec);
    var_min = (fabs(model.dec) - degrees)*60.0;
    if (model.dec > 0.0) SPRINTF(p,"%s", "-");
    SPRINTF(p,"%d:", degrees);
    SPRINTF(p,"%g:", var_min);
  } else {
    return FALSE;
  }

  /* signed latitude */

  if (*lat_ns == 'S') SPRINTF(p,"%s", "-");
  SPRINTF(p,"%d:", lat_deg);
  SPRINTF(p,"%g:", lat_min);

  /* signed longitude */

  if (*lon_ew == 'E') SPRINTF(p,"%s", "-");	/* bass ackwards convention! */
  SPRINTF(p,"%d:", lon_deg);
  SPRINTF(p,"%g:", lon_min);

  /* navaid type */

  SPRINTF(p,"%s:", "INT");

  /* comment string */

  if (rec->loc_idTfacility_typeTbearing_dist_1st_navaid[0] != '\0') {
    navaid_id = strtok(rec->loc_idTfacility_typeTbearing_dist_1st_navaid, "*");
    radial_dist = strtok(NULL, "*");
    radial_dist = strtok(NULL, "*");
    SPRINTF(p,"%s", navaid_id);
    SPRINTF(p," %s", radial_dist);
  }

  if (rec->loc_idTfacility_typeTbearing_dist_2nd_navaid[0] != '\0') {
    navaid_id = strtok(rec->loc_idTfacility_typeTbearing_dist_2nd_navaid, "*");
    radial_dist = strtok(NULL, "*");
    radial_dist = strtok(NULL, "*");
    SPRINTF(p,", %s", navaid_id);
    SPRINTF(p," %s", radial_dist);
  }

  if (rec->loc_idTfacility_typeTbearing_dist_3rd_navaid[0] != '\0') {
    navaid_id = strtok(rec->loc_idTfacility_typeTbearing_dist_3rd_navaid, "*");
    radial_dist = strtok(NULL, "*");
    radial_dist = strtok(NULL, "*");
    SPRINTF(p,", %s", navaid_id);
    SPRINTF(p," %s", radial_dist);
  }

  if (rec->loc_idTfacility_typeTbearing_dist_4th_navaid[0] != '\0') {
    navaid_id = strtok(rec->loc_idTfacility_typeTbearing_dist_4th_navaid, "*");
    radial_dist = strtok(NULL, "*");
    radial_dist = strtok(NULL, "*");
    SPRINTF(p,", %s", navaid_id);
    SPRINTF(p," %s", radial_dist);
  }

  /* normal exit */

  *p = '\0';

  fprintf(fpoutput, "%s\n", outbuf);

  return TRUE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_nav_ils(char **tok_str)
{

  /* no ils identifiers were provided, and fplan needs one --> data unusable */

  return FALSE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_nav_nav(char **tok_str)
{

  nav_t *rec = (void *) tok_str;

  char *p = outbuf;

  int nscan;
  int degrees;
  int lat_deg;
  int lon_deg;
  int minutes;
  double lat_min;
  double lon_min;
  double var_min;
  double seconds;
  char lat_ns[2];
  char lon_ew[2];
  char *keyword;

  /* facility id */

  if (rec->facility_id_wo_spec_chars[0] != '\0') {
    if (strlen(rec->facility_id_wo_spec_chars) > 5)
      rec->facility_id_wo_spec_chars[5] = '\0';
    SPRINTF(p,"%s:", rec->facility_id_wo_spec_chars);
  } else {
    return FALSE;
  }

  /* facility name */

  if (strlen(rec->navaid_name) > 32)
    rec->navaid_name[32] = '\0';
  SPRINTF(p,"%s:", rec->navaid_name);

  /* frequency */

  SPRINTF(p,"%s:", rec->frequency);

  /* elevation (feet) */

  SPRINTF(p,"%s:", rec->elevation_feet);

  /* scan latitude */

  nscan = sscanf(rec->latitude, "%d-%d-%lf%1[NS]",
                 &lat_deg, &minutes, &seconds, lat_ns);

  if (nscan == 4) {
    lat_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* scan longitude */

  nscan = sscanf(rec->longitude, "%d-%d-%lf%1[EW]",
                 &lon_deg, &minutes, &seconds, lon_ew);

  if (nscan == 4) {
    lon_min = (double) minutes + (seconds/60.0);
  } else {
    return FALSE;
  }

  /* signed magnetic variation */

  model.latitude = ((*lat_ns == 'N')? +1.0: -1.0)*(lat_deg + lat_min/60.0);
  model.longitude = ((*lon_ew == 'E')? +1.0: -1.0)*(lon_deg + lon_min/60.0);
  if (sscanf(rec->elevation_feet, "%lf", &model.altitude) == 1) {
    model.altitude = FEET2KM(model.altitude);
  } else {
    model.altitude = 1.0;
  }

  if (field_model(&model)) {
    degrees = fabs(model.dec);
    var_min = (fabs(model.dec) - degrees)*60.0;
    if (model.dec > 0.0) SPRINTF(p,"%s", "-");
    SPRINTF(p,"%d:", degrees);
    SPRINTF(p,"%g:", var_min);
  } else {
    return FALSE;
  }

  /* signed latitude */

  if (*lat_ns == 'S') SPRINTF(p,"%s", "-");
  SPRINTF(p,"%d:", lat_deg);
  SPRINTF(p,"%g:", lat_min);

  /* signed longitude */

  if (*lon_ew == 'E') SPRINTF(p,"%s", "-");	/* bass ackwards convention! */
  SPRINTF(p,"%d:", lon_deg);
  SPRINTF(p,"%g:", lon_min);

  /* navaid type */

  if (strcmp(rec->facility_type,"VOR") == 0) {
    keyword = "VOR";
  } else if ((strcmp(rec->facility_type,"VORTAC") == 0) ||
             (strcmp(rec->facility_type,"VOR/DME") == 0)) {
    keyword = "DME";
  } else if (strcmp(rec->facility_type,"TACAN") == 0) {
    keyword = "TAC";
  } else if ((strcmp(rec->facility_type,"NDB") == 0) ||
             (strcmp(rec->facility_type,"NDB/DME") == 0) ||
             (strcmp(rec->facility_type,"UHF NDB") == 0) ||
             (strcmp(rec->facility_type,"MARINE NDB") == 0)) {
    keyword = "NDB";
  } else {
    return FALSE;
  }

  SPRINTF(p,"%s:", keyword);

  /* comment string */

  if (rec->class[0] != '\0') {
    SPRINTF(p,"%s", rec->class);
  }

  if (rec->controlling_fss_name[0] != '\0') {
    rec->controlling_fss_name = strcap(rec->controlling_fss_name);
    SPRINTF(p,", %s FSS", rec->controlling_fss_name);
  }

  /* normal exit */

  *p = '\0';

  fprintf(fpoutput, "%s\n", outbuf);

  return TRUE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_nav_rwy(char **tok_str)
{

  /* none of this information is read by fplan */

  return FALSE;

}

/* -------------------------------------------------------------------------- */

bool_t write_nav(char **tok_str)
{

  bool_t rc = FALSE;

  switch (db_type) {

    case DB_APT:
      rc = write_nav_apt(tok_str);
      break;

    case DB_FIX:
      rc = write_nav_fix(tok_str);
      break;

    case DB_ILS:
      rc = write_nav_ils(tok_str);
      break;

    case DB_NAV:
      rc = write_nav_nav(tok_str);
      break;

    case DB_RWY:
      rc = write_nav_rwy(tok_str);
      break;

  }

  /* return result */

  return rc;

}

/* -------------------------------------------------------------------------- */

